// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

#pragma once

#include "exec/exec_node.h"
#include "exprs/runtime_filter.h"

namespace doris::vectorized {

class RuntimeFilterConsumerNode : public ExecNode {
public:
    RuntimeFilterConsumerNode(ObjectPool* pool, const TPlanNode& tnode, const DescriptorTbl& descs);
    ~RuntimeFilterConsumerNode() override = default;

    Status init(const TPlanNode& tnode, RuntimeState* state = nullptr) override;

    // Try append late arrived runtime filters.
    // Return num of filters which are applied already.
    Status try_append_late_arrival_runtime_filter(int* arrived_rf_num);

    bool runtime_filters_are_ready_or_timeout();

protected:
    // Register and get all runtime filters at Init phase.
    Status _register_runtime_filter();
    // Get all arrived runtime filters at Open phase.
    Status _acquire_runtime_filter(bool wait = true);
    // Append late-arrival runtime filters to the vconjunct_ctx.
    Status _append_rf_into_conjuncts(const VExprSPtrs& vexprs);

    void _prepare_rf_timer(RuntimeProfile* profile);

    // For runtime filters
    struct RuntimeFilterContext {
        RuntimeFilterContext() : apply_mark(false), runtime_filter(nullptr) {}
        RuntimeFilterContext(IRuntimeFilter* rf) : apply_mark(false), runtime_filter(rf) {}
        // set to true if this runtime filter is already applied to vconjunct_ctx_ptr
        bool apply_mark;
        IRuntimeFilter* runtime_filter;
    };

    RuntimeState* _state;

    std::vector<RuntimeFilterContext> _runtime_filter_ctxs;

    std::vector<TRuntimeFilterDesc> _runtime_filter_descs;
    // Set to true if the runtime filter is ready.
    std::vector<bool> _runtime_filter_ready_flag;
    doris::Mutex _rf_locks;
    phmap::flat_hash_set<VExprSPtr> _rf_vexpr_set;
    // True means all runtime filters are applied to scanners
    bool _is_all_rf_applied = true;
    bool _blocked_by_rf = false;

    RuntimeProfile::Counter* _acquire_runtime_filter_timer = nullptr;
};

} // namespace doris::vectorized